/*
 * audio.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

audio_t *audio_init(audio_t *a, plugin_t *p)
{
  a->p=p;

  if (NULL==(a->frame=avcodec_alloc_frame()))
    goto error;

  avcodec_get_frame_defaults(a->frame);

  if (NULL==queue_init(&a->q,p,"Audio",p->ps.audio.queue_size))
    goto error;

  if (NULL==samples_init(&a->s))
    goto error;

  a->chain=NULL;
  ar_init(&a->ar,a);

  return a;
error:
  audio_cleanup(a);

  return NULL;
}

audio_t *audio_cleanup(audio_t *a)
{
  samples_cleanup(&a->s);
  queue_cleanup(&a->q);

  if (NULL!=a->frame)
    av_free(a->frame);

  return a;
}

audio_t *audio_realloc(audio_t *a, track_t *t)
{
  int sox_needed=track_sox_needed(t,&a->in,&a->out);

  a->st=t->ffi.ast;

  if (NULL==samples_realloc(&a->s,t,a->out.nch,a->out.bps))
    goto error;

  if (sox_needed&&NULL==(a->chain=ffsox_chain(a)))
    goto error;

  return a;
error:
  return NULL;
}

void audio_reset(audio_t *a)
{
  prefs_t *ps=&a->p->ps;

  avcodec_get_frame_defaults(a->frame);
  samples_clear(&a->s);
  queue_reset(&a->q,ps->audio.queue_size);
}

audio_t *audio_run(audio_t *a)
{
  a->hThread=CreateThread(
    NULL,   // _In_opt_   LPSECURITY_ATTRIBUTES lpThreadAttributes,
    0,      // _In_       SIZE_T dwStackSize,
    audio_thread,
            // _In_       LPTHREAD_START_ROUTINE lpStartAddress,
    a,      // _In_opt_   LPVOID lpParameter,
    0,      // _In_       DWORD dwCreationFlags,
    NULL    // _Out_opt_  LPDWORD lpThreadId
  );

  return NULL==a->hThread?NULL:a;
}

void audio_wait(audio_t *a)
{
  if (NULL!=a->hThread) {
    WaitForSingleObject(a->hThread,INFINITE);
    CloseHandle(a->hThread);
    a->hThread=NULL;
  }
}

void audio_stop(audio_t *a)
{
  if (NULL!=a->chain) {
    sox_delete_effects_chain(a->chain);
    a->chain=NULL;
  }

  a->st=NULL;
}

void audio_kill(audio_t *a)
{
  samples_kill(&a->s);
}

DWORD WINAPI audio_thread(LPVOID lpParam)
{
  audio_t *a=lpParam;
  ar_t *ar=&a->ar;

  ar_first(ar);

  if (NULL==a->chain) {
#if defined (DEBUG) // {
    fprintf(stderr,"NATIVE ...\n");
#endif // }

    while (ar_valid(ar)) {
      samples_copy(&a->s,ar);
      ar_next(ar);
    }
  }
  else {
#if defined (DEBUG) // {
    fprintf(stderr,"SOX ...\n");
#endif // }

    sox_flow_effects(a->chain,NULL,NULL);
  }

  // stop samples
  WaitForSingleObject(a->s.hMutex,INFINITE);

  if (STATE_RUN==a->s.state)
    a->s.state=STATE_EOF;

  SetEvent(a->s.hEventRead);
  ReleaseMutex(a->s.hMutex);

  return 0;
}
