/*
 * ausdio_reader.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

void ar_init(ar_t *ar, audio_t *a)
{
  ar->state=AR_STATE_READ;
  ar->a=a;
  ar->p=a->p;
  ar->q=&a->q;
  ar->frame=a->frame;
  ar->dc=NULL;
  ar->touched=0;
  ZERO(&ar->pkt);
}

void ar_first(ar_t *ar)
{
  audio_t *a=ar->a;

  ar->state=AR_STATE_READ;
  ar->dc=a->st->dc;
  ar->touched=0;
  ZERO(&ar->pkt);
  ar_next(ar);
}

static int ar_decode(ar_t *ar, int next_state)
{
  int got_frame=0;

  if (avcodec_decode_audio4(ar->dc,ar->frame,&got_frame,&ar->pkt)<0)
	// ignore the error and continue.
    //next_state=AR_STATE_ERROR;
	return -1;

  if (AR_STATE_READ==ar->state)
    av_free_packet(&ar->pkt);

  if (AR_STATE_ERROR==next_state||!got_frame) {
    ar->state=next_state;
    return -1;
  }
  else {
    if (!ar->touched) {
      audio_t *a=ar->a;
      samples_t *s=&a->s;
      ffi_st_t *ast=a->st;
      double ts=av_frame_get_best_effort_timestamp(a->frame);
      double time=1000.0*av_q2d(ast->st->time_base)*ts;
      double length=track_get_length(s->t);

      if (time<=length)
        s->time=time;

      ar->touched=1;
    }

    return 0;
  }
}

void ar_next(ar_t *ar)
{
  while (ar_valid(ar)) {
    if (STATE_KILL==ar->p->state)
      ar->state=AR_STATE_KILL;

    switch (ar->state) {
    case AR_STATE_READ:
      if (0!=queue_read(ar->q,&ar->pkt)) {
        ZERO(&ar->pkt);
        ar->state=AR_STATE_FLUSH;
        continue;
      }

      if (0==ar_decode(ar,AR_STATE_READ))
        return;

      continue;
    case AR_STATE_FLUSH:
      if (0==ar_decode(ar,AR_STATE_DONE))
        return;

      continue;
    default:
      return;
    }
  }
}
