/*
 * prefdef.h
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PREFDEF_H
#define PREFDEF_H

#include "resources.h"
#include "prefs.h"

///////////////////////////////////////////////////////////////////////////////
#define SLIDERI(imin,imax) \
  (slider_t){ .i.min=imin,.i.max=imax }
#define SLIDERX(xmin,xmax,xstep) \
  (slider_t){ .x.min=xmin,.x.max=xmax,.x.step=xstep }

///////////////////////////////////////////////////////////////////////////////
#define GENERAL "general"

#define SIZEOF(T,f) \
  (sizeof (((T *)NULL)->f))

#define PREFDEF(group,name) \
  #group,#name,offsetof(prefs_t,group.name),SIZEOF(prefs_t,group.name)

#define PREFS_INT(group,name) \
  PREFDEF(group,name),PREF_INT
#define SLIDER_INT(group,name,id,min,max) \
  PREFS_INT(group,name),id,prefs_slider,.slider=&SLIDERI(min,max)

#define PREFS_DOUBLE(group,name) \
  PREFDEF(group,name),PREF_DOUBLE
#define SLIDER_DOUBLE(group,name,id,min,max,step) \
  PREFS_DOUBLE(group,name),id,prefs_slider,.slider=&SLIDERX(min,max,step)
 
#define PREFS_STRING(group,name) \
  PREFDEF(group,name),PREF_STRING

///////////////////////////////////////////////////////////////////////////////
#define GENERAL_PDS() (prefdef_t []){ \
  /* general */ \
  { PREFS_INT(general,bps),IDC_COMBO_WINAMP_BPS, \
      prefs_combo,.combo=(combo_t []){ \
        { "16",.i=16 }, \
        { "24",.i=24 }, \
        { NULL } \
      }}, \
  { PREFS_INT(general,vid),IDC_CHECKBOX_WINAMP_VID,prefs_check }, \
  { PREFS_INT(general,vis),IDC_CHECKBOX_WINAMP_VIS,prefs_check }, \
  /* rg */ \
  { PREFS_DOUBLE(rg,weight),IDC_COMBO_RG_WEIGHT, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.x=-1.0 }, \
        { "Album",.x=0.0 }, \
        { "25%",.x=0.25 }, \
        { "50%",.x=0.5 }, \
        { "75%",.x=0.75 }, \
        { "Track",.x=1.0 }, \
        { NULL } \
      }}, \
  { SLIDER_DOUBLE(rg,preamp,IDC_SLIDER_RG_PREAMP,-12.0,12.0,0.1) }, \
  { PREFS_INT(rg,comment),IDC_CHECKBOX_RG_COMMENT,prefs_check }, \
}

enum { PREFS_GENERAL_N=(sizeof GENERAL_PDS())/(sizeof GENERAL_PDS()[0]) };

typedef struct prefs_general {
  prefs_t *ps;
  prefgui_t pg[PREFS_GENERAL_N];

  struct {
    prefgui_t *preamp;
  } rg;
} prefs_general_t;

///////////////////////////////////////////////////////////////////////////////
#define AUDIO_PDS() (prefdef_t []){ \
  /* audio */ \
  { SLIDER_INT(audio,queue_size,IDC_SLIDER_AUDIO_QUEUE_SIZE,0,15) }, \
  /* sox */ \
  /* { PREFS_INT(sox,off),IDC_CHECKBOX_SOX_OFF,prefs_check }, */ \
  { PREFS_STRING(sox,mode),IDC_COMBO_SOX_MODE, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.s="off" }, \
        { "Automatic",.s="auto" }, \
        { "On",.s="on" }, \
        { NULL } \
      }}, \
  { PREFS_INT(sox,rate),IDC_COMBO_SOX_RATE, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.i=0 }, \
        { "44100 Hz",.i=44100 }, \
        { "48000 Hz",.i=48000 }, \
        { "88200 Hz",.i=88200 }, \
        { "96000 Hz",.i=96000 }, \
        { "176400 Hz",.i=176400 }, \
        { "192000 Hz",.i=192000 }, \
        { NULL } \
      }}, \
  { PREFS_STRING(sox,rate_q),IDC_COMBO_SOX_RATE_Q, \
      prefs_combo,.combo=(combo_t []){ \
        { "Default",.s="" }, \
        { "Quick",.s="-q" }, \
        { "Low",.s="-l" }, \
        { "Medium",.s="-m" }, \
        { "High",.s="-h" }, \
        { "Very High",.s="-v" }, \
        { NULL } \
      }}, \
  { PREFS_INT(sox,rate_min),IDC_CHECKBOX_SOX_RATE_MIN,prefs_check }, \
  { PREFS_INT(sox,rate_mul),IDC_CHECKBOX_SOX_RATE_MUL,prefs_check }, \
  { PREFS_STRING(sox,dither),IDC_COMBO_SOX_DITHER, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.s="off" }, \
        { "Default",.s="" }, \
        { "Sloped TPDF",.s="-S" }, \
        { "Noise Shaping",.s="-s" }, \
        { "Lipshitz Filter",.s="-f lipshitz" }, \
        { "F-Weighted Filter",.s="-f f-weighted" }, \
        { "Modified E-Weighted Filter",.s="-f modified-e-weighted" }, \
        { "Improved E-Weighted Filter",.s="-f improved-e-weighted" }, \
        { "Gesemann Filter",.s="-f gesemann" }, \
        { "Shibata Filter",.s="-f shibata" }, \
        { "Low Shibata Filter",.s="-f low-shibata" }, \
        { "High Shibata Filter",.s="-f high-shibata" }, \
        { NULL } \
      }}, \
}

enum { PREFS_AUDIO_N=(sizeof AUDIO_PDS())/(sizeof AUDIO_PDS()[0]) };

typedef struct prefs_audio {
  prefs_t *ps;
  prefgui_t pg[PREFS_AUDIO_N];

  struct {
    prefgui_t *queue_size;
  } audio;
} prefs_audio_t;

///////////////////////////////////////////////////////////////////////////////
#define VIDEO_PDS() (prefdef_t []){ \
  /* video */ \
  { SLIDER_INT(video,queue_size,IDC_SLIDER_VIDEO_QUEUE_SIZE,0,15) }, \
  { SLIDER_DOUBLE(video,min_drift,IDC_SLIDER_VIDEO_MIN_DRIFT, \
      -2000.0,0.0,10.0) }, \
  { SLIDER_DOUBLE(video,max_drift,IDC_SLIDER_VIDEO_MAX_DRIFT, \
      0.0,16000.0,10.0) }, \
}

enum { PREFS_VIDEO_N=(sizeof VIDEO_PDS())/(sizeof VIDEO_PDS()[0]) };

typedef struct prefs_video {
  prefs_t *ps;
  prefgui_t pg[PREFS_VIDEO_N];

  struct {
    prefgui_t *queue_size;
    prefgui_t *min_drift;
    prefgui_t *max_drift;
  } video;
} prefs_video_t;

#endif // PREFDEF_H
