/*
 * prefs_general.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "prefdef.h"

#define PREFS_GENERAL

///////////////////////////////////////////////////////////////////////////////
#if ! defined (FULL) // {
#define EXTS "FLAC;MKV;OGG;WAV;WEBM;WV"
#else // } {
#define EXTS "FLAC;MKV;OGG;WAV;WEBM;WV;MP3;MP4;FLV;WMV;VOB;AVI;M2TS"
#endif // }

///////////////////////////////////////////////////////////////////////////////
typedef prefs_general_t priv_t;

static prefdef_t pds[]=GENERAL_PDS();
static prefdef_t *mpd=pds+PREFS_GENERAL_N;

///////////////////////////////////////////////////////////////////////////////
static void display2plugin(buf_t *display, buf_t *plugin)
{
  char *rp=display->rep;
  char *mp=display->rep;

  plugin->wp=plugin->rep;

  for (;;) {
    if (';'==*mp||'\0'==*mp) {
      if (NULL==buf_append_n(plugin,rp,mp-rp))
        break;

      ++plugin->wp;

      if (NULL==buf_append_n(plugin,rp,mp-rp))
        break;

      ++plugin->wp;

      if ('\0'==*mp)
        break;
      else
        rp=++mp;
    }
    else
      ++mp;
  }
}

static void prefs_read_exts(prefs_t *ps)
{
  char *path=PS_UTF8(ps->path);
  int n;

  ps->exts.display.wp=ps->exts.display.rep;
  n=ps->exts.display.mp-ps->exts.display.wp-2;
  n=GetPrivateProfileStringA(GENERAL,"exts",EXTS,ps->exts.display.wp,n,path);
  ps->exts.display.wp+=n;
  ps->exts.display.wp[0]='\0';
  ps->exts.display.wp[1]='\0';
  display2plugin(&ps->exts.display,&ps->exts.plugin);
}

static void prefs_write_exts(prefs_t *ps, HWND hDlg)
{
  char *path=PS_UTF8(ps->path);
  int n;

  ps->exts.display.wp=ps->exts.display.rep;
  n=ps->exts.display.mp-ps->exts.display.wp-2;
  n=GetDlgItemTextA(hDlg,IDC_EDIT_WINAMP_EXTS,ps->exts.display.wp,n);
  ps->exts.display.wp+=n;
  ps->exts.display.wp[0]='\0';
  ps->exts.display.wp[1]='\0';
  WritePrivateProfileStringA(GENERAL,"exts",ps->exts.display.rep,path);
  display2plugin(&ps->exts.display,&ps->exts.plugin);
}

///////////////////////////////////////////////////////////////////////////////
void prefs_general_load(prefs_t *ps)
{
  prefdef_t *pd;

  prefs_read_exts(ps);

  for (pd=pds;pd<mpd;++pd)
    prefs_read(ps,pd);
}

///////////////////////////////////////////////////////////////////////////////
#define PREFS_SYNC_SLIDER
#include "slider.h"

static void priv_init(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  SetDlgItemTextA(hDlg,IDC_EDIT_WINAMP_EXTS,ps->exts.display.rep);

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pd->init) {
      pd->init(pg,pd,ps,hDlg);

      switch (pg->id) {
#define PREFS_INIT_SLIDER
#include "slider.h"
      default:
        break;
      }
    }
  }
}

static void priv_save(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  prefs_write_exts(ps,hDlg);

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pg->vmt) {
      pg->vmt->get(pg,ps,hDlg);
      prefs_write(ps,pd);
    }
  }
}

static void priv_display(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  SetDlgItemTextA(hDlg,IDC_EDIT_WINAMP_EXTS,EXTS);

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pg->vmt)
      pg->vmt->set(pg,ps,hDlg);
  }
}

///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK GeneralDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  priv_t *p=WM_INITDIALOG==uMsg?(void *)lParam
      :(void *)GetWindowLongPtr(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
	SetWindowLongPtr(hDlg,GWLP_USERDATA,lParam);
    priv_init(p,p->ps,hDlg);
    return TRUE;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      priv_save(p,p->ps,hDlg);
      return TRUE;
    case IDC_BUTTON_DEFAULT:
      priv_display(p,(prefs_t *)lParam,hDlg);
      return TRUE;
    default:
	  break;
    }

    break;
  case WM_NOTIFY:
    switch (LOWORD(wParam)) {
#define PREFS_NOTIFY_SLIDER
#include "slider.h"
    default:
      break;
    }

    break;
  default:
    break;
  }

  return FALSE;
}
