/*
 * pref_slider.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "prefs.h"
#include <math.h>
#include <commctrl.h>

static prefvmt_t vmt;

prefgui_t *prefs_slider(prefgui_t *pg, prefdef_t *pd, prefs_t *ps, HWND hDlg)
{
  slider_t *s=pd->slider;

  pg->vmt=&vmt;
  pg->pd=pd;
  pg->id=pd->id;

  switch (pd->type) {
  case PREF_INT:
    SendDlgItemMessage(hDlg,pg->id,TBM_SETRANGEMIN,TRUE,0);
    SendDlgItemMessage(hDlg,pg->id,TBM_SETRANGEMAX,TRUE,s->i.max-s->i.min);
    break;
  case PREF_DOUBLE:
    SendDlgItemMessage(hDlg,pg->id,TBM_SETRANGEMIN,TRUE,0);
    SendDlgItemMessage(hDlg,pg->id,TBM_SETRANGEMAX,TRUE,
        round((s->x.max-s->x.min)/s->x.step));
    break;
  default:
    break;
  }

  vmt.set(pg,ps,hDlg);

  return pg;
}

static void set(prefgui_t *pg, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  slider_t *s=pd->slider;

  switch (pd->type) {
  case PREF_INT:
    SendDlgItemMessage(hDlg,pg->id,TBM_SETPOS,TRUE,
        pref_int(pd,ps)-s->i.min);
    break;
  case PREF_DOUBLE:
    SendDlgItemMessage(hDlg,pg->id,TBM_SETPOS,TRUE,
        round((pref_double(pd,ps)-s->x.min)/s->x.step));
    break;
  default:
    break;
  }
}

static void get(prefgui_t *pg, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd=pg->pd;

  switch (pd->type) {
  case PREF_INT:
    pref_int(pd,ps)=vmt.get_int(pg,hDlg);
	break;
  case PREF_DOUBLE:
    pref_double(pd,ps)=vmt.get_double(pg,hDlg);
	break;
  default:
    break;
  }
}

static int get_int(prefgui_t *pg, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  slider_t *s=pd->slider;
  int pos=SendDlgItemMessage(hDlg,pg->id,TBM_GETPOS,0,0);

  return PREF_INT==pd->type?s->i.min+pos:0;
}

static double get_double(prefgui_t *pg, HWND hDlg)
{
  prefdef_t *pd=pg->pd;
  slider_t *s=pd->slider;
  double pos=SendDlgItemMessage(hDlg,pg->id,TBM_GETPOS,0,0);

  return PREF_DOUBLE==pd->type?s->x.min+pos*s->x.step:0.0;
}

static prefvmt_t vmt={
  .set=set,
  .get=get,
  .get_int=get_int,
  .get_double=get_double
};
