/*
 * prefs_tabs.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <windows.h>
#include <commctrl.h>
#include "prefs.h"

DLGTEMPLATE *load_template(HMODULE hMod, int id)
{
  HRSRC hRsrc;
  HGLOBAL hGlb;
  DLGTEMPLATE *res;

  if (NULL==(hRsrc=FindResource(hMod,MAKEINTRESOURCE(id),RT_DIALOG))) {
#if defined (DEBUG) // {
    fprintf(stderr,"Error finding resource.\n");
#endif // }
    return NULL;
  }

  if (NULL==(hGlb=LoadResource(hMod,hRsrc))) {
#if defined (DEBUG) // {
    fprintf(stderr,"Error loading resource.\n");
#endif // }
    return NULL;
  }

  if (NULL==(res=(DLGTEMPLATE *)LockResource(hGlb))) {
#if defined (DEBUG) // {
    fprintf(stderr,"Error locking resource.\n");
#endif // }
    return NULL;
  }

  return res;
}

tabs_t *tabs_init(tabs_t *tc, HWND hDlg, int id, int n, tcp_t *tcp,
    HWND *hPages, char *page)
{
  HMODULE hMod=GetModuleHandleA(MODULE);
  HWND hTab=GetDlgItem(hDlg,id);
  TCITEM tci;
  tcp_t *cur;
  int i;
  DLGTEMPLATE *res;

  tc->id=id;
  tc->hPages=hPages;

  ZERO(&tci);
  tci.mask=TCIF_TEXT;

  for (cur=tcp,i=0;NULL!=cur->title;++cur,++i) {
    if (NULL==(res=load_template(hMod,cur->id)))
      continue;

    tci.pszText=cur->title;
    TabCtrl_InsertItem(hTab,cur-tcp,&tci);

    hPages[i]=CreateDialogIndirectParamW(
      hMod,                     // _In_opt_  HINSTANCE hInstance,
      res,                      // _In_      LPCDLGTEMPLATE lpTemplate,
      hTab,                     // _In_opt_  HWND hWndParent,
      cur->proc,                // _In_opt_  DLGPROC lpDialogFunc,
      (LPARAM)(page+cur->offs)	// _In_      LPARAM lParamInit
    ); 

    SetWindowPos(hPages[i],HWND_TOP,2,22,0,0,SWP_NOSIZE);

    if (n==i) {
      TabCtrl_SetCurSel(hTab,n);
      tc->hCurPage=tc->hPages[n];
      ShowWindow(tc->hCurPage,SW_SHOW);
    }
    else
      ShowWindow(hPages[i],SW_HIDE);
  }

  return tc;
}

void tabs_set_page(tabs_t *tc, HWND hDlg, int i)
{ 
  HWND hTab=GetDlgItem(hDlg,tc->id);
 
  if (tc->hCurPage!=NULL) 
    ShowWindow(tc->hCurPage,SW_HIDE);
 
  tc->hCurPage=tc->hPages[i];
  ShowWindow(tc->hCurPage,SW_SHOW);
  TabCtrl_SetCurSel(hTab,i);
}

void tabs_change(tabs_t *tc, HWND hDlg)
{ 
  HWND hTab=GetDlgItem(hDlg,tc->id);
  int i=TabCtrl_GetCurSel(hTab); 
 
  if (tc->hCurPage!=NULL) 
    ShowWindow(tc->hCurPage,SW_HIDE);
 
  tc->hCurPage=tc->hPages[i];
  ShowWindow(tc->hCurPage,SW_SHOW);
}
